/* SPS2WRAP.CPP - Programmed by Jonathan Hobson */

#include "sps2wrap.h"
#include "ps2defines.h"
#include "dma.h"

CSPS2Manager SPS2Manager_SingletonInit;

CSPS2Manager::CSPS2Manager()
{
}

CSPS2Manager::~CSPS2Manager()
{
	sps2UScreenShutdown();
	sps2Release(m_iSPS2Desc);
}

void CSPS2Manager::Initialise(const int iAllocPages)
{
	m_iNumPages = iAllocPages;

	m_iSPS2Desc = sps2Init();

	if(m_iSPS2Desc < 0)
	{
		fprintf(stderr, "Failed to initialise SPS2 library." \
				"Please check that the module is loaded.\n");
		exit(-1);
	}

	int iVPU0; iVPU0 = open("/dev/ps2vpu0", O_RDWR);
	int iVPU1; iVPU1 = open("/dev/ps2vpu1", O_RDWR);

	m_pMemCached = sps2Allocate(iAllocPages * 4096,
			SPS2_MAP_BLOCK_4K | SPS2_MAP_CACHED, m_iSPS2Desc);
	m_pMemUncached = sps2Remap(m_pMemCached, SPS2_MAP_UNCACHED, m_iSPS2Desc);

	m_iaFreeMemInPage = new int[m_iNumPages];

	for(int iPage = 0; iPage < m_iNumPages; iPage++)
	{
		m_iaFreeMemInPage[iPage] = 256; 
	}

	sps2UScreenInit(0);
}

void CSPS2Manager::Allocate(CDMAMem & Memory, int iQuadWords)
{
	assert(iQuadWords <= 256);

	for(int iPage = 0; iPage < m_iNumPages; iPage++)
	{
		if(m_iaFreeMemInPage[iPage] >= iQuadWords)
		{
			int FirstQW = 256 - m_iaFreeMemInPage[iPage];
			m_iaFreeMemInPage[iPage] -= iQuadWords;
			void * pCached = &((char *)m_pMemCached->pvStart)[iPage * 4096 + (FirstQW * 16)];
			void * pUncached = &((char *)m_pMemUncached->pvStart)[iPage * 4096 + (FirstQW * 16)];
			Memory.Set(iQuadWords, pCached, pUncached, sps2GetPhysicalAddress(pCached, m_pMemCached));

			return;
		}
	}

	assert(!"Not enough memory");
}

void CSPS2Manager::BeginScene(int R, int G, int B)
{
	int x0 = (2048 - (sps2UScreenGetWidth() >> 1)) << 4;
	int y0 = (2048 - (sps2UScreenGetHeight() >> 1)) << 4;
	int x1 = (2048 + (sps2UScreenGetWidth() >> 1)) << 4;
	int y1 = (2048 + (sps2UScreenGetHeight() >> 1)) << 4;

	VIFDynamicDMA.StartDirect();

	VIFDynamicDMA.Add128(GS_GIFTAG_BATCH(6, 1, 0, 0, 
						GIF_FLG_PACKED, GS_BATCH_1(GIF_REG_A_D)));

	VIFDynamicDMA.Add64(TEST_SET(0, 0, 0, 0, 0, 0, 1, 1));
	VIFDynamicDMA.Add64(TEST_1);

	VIFDynamicDMA.Add64(PRIM_SET(0x6, 0, 0, 0, 0, 0, 0, 0, 0));
	VIFDynamicDMA.Add64(PRIM);

	VIFDynamicDMA.Add64(RGBAQ_SET(R, G, B, 0x80, 0x3f800000));
	VIFDynamicDMA.Add64(RGBAQ);

	VIFDynamicDMA.Add64(XYZ2_SET(x0, y0, 0));
	VIFDynamicDMA.Add64(XYZ2);

	VIFDynamicDMA.Add64(XYZ2_SET(x1, y1, 0));
	VIFDynamicDMA.Add64(XYZ2);

	VIFDynamicDMA.Add64(TEST_SET(0, 0, 0, 0, 0, 0, 1, 3));
	VIFDynamicDMA.Add64(TEST_1);

	VIFDynamicDMA.EndDirect();
}

